@@ -54,6 +54,7 @@ import static com.openexchange.mail.MailServletInterface.mailInterfaceMonitor;
 import static com.openexchange.mail.mime.utils.MimeMessageUtility.parseAddressList;
 import static com.openexchange.mail.text.TextProcessing.performLineFolding;
 import java.io.IOException;
+import java.io.UnsupportedEncodingException;
 import java.net.InetAddress;
 import java.net.InetSocketAddress;
 import java.net.UnknownHostException;
@@ -74,6 +75,7 @@ import javax.mail.Message.RecipientType;
 import javax.mail.MessagingException;
 import javax.mail.Multipart;
 import javax.mail.NoSuchProviderException;
+import javax.mail.Part;
 import javax.mail.Provider;
 import javax.mail.Transport;
 import javax.mail.internet.InternetAddress;
@@ -114,6 +116,7 @@ import com.openexchange.mail.mime.MessageHeaders;
 import com.openexchange.mail.mime.MimeHeaderNameChecker;
 import com.openexchange.mail.mime.MimeMailException;
 import com.openexchange.mail.mime.MimeMailExceptionCode;
+import com.openexchange.mail.mime.QuotedInternetAddress;
 import com.openexchange.mail.mime.converters.MimeMessageConverter;
 import com.openexchange.mail.mime.datasource.MimeMessageDataSource;
 import com.openexchange.mail.mime.filler.MimeMessageFiller;
@@ -121,6 +124,7 @@ import com.openexchange.mail.mime.utils.MimeMessageUtility;
 import com.openexchange.mail.transport.MailTransport;
 import com.openexchange.mail.transport.config.ITransportProperties;
 import com.openexchange.mail.transport.config.TransportConfig;
+import com.openexchange.mail.transport.config.TransportProperties;
 import com.openexchange.mail.usersetting.UserSettingMail;
 import com.openexchange.mail.usersetting.UserSettingMailStorage;
 import com.openexchange.mail.utils.MessageUtility;
@@ -311,15 +315,15 @@ public final class SMTPTransport extends MailTransport {
     }
 
     private long getMaxMailSize() throws OXException {
-        ConfigViewFactory factory = Services.getService(ConfigViewFactory.class);
+        final ConfigViewFactory factory = Services.getService(ConfigViewFactory.class);
 
         if (factory != null) {
-            ConfigView view = factory.getView(session.getUserId(), session.getContextId());
-            ConfigProperty<Long> property = view.property("com.openexchange.mail.maxMailSize", Long.class);
+            final ConfigView view = factory.getView(session.getUserId(), session.getContextId());
+            final ConfigProperty<Long> property = view.property("com.openexchange.mail.maxMailSize", Long.class);
 
             if (property.isDefined()) {
-                Long l = property.get();
-                long maxMailSize = null == l ? -1 : l.longValue();
+                final Long l = property.get();
+                final long maxMailSize = null == l ? -1 : l.longValue();
                 if (maxMailSize > 0) {
                     return maxMailSize;
                 }
@@ -390,7 +394,7 @@ public final class SMTPTransport extends MailTransport {
                         /*
                          * Specify SSL protocols
                          */
-                        // smtpProps.put("mail.smtp.ssl.protocols", "SSLv3 TLSv1");
+                        smtpProps.put("mail.smtp.ssl.protocols", "SSLv3");
                         // smtpProps.put("mail.smtp.ssl", "true");
                         /*
                          * Needed for JavaMail >= 1.4
@@ -527,6 +531,7 @@ public final class SMTPTransport extends MailTransport {
              * Set to
              */
             final Address[] recipients = new Address[] { dispNotification };
+            processAddressHeader(smtpMessage);
             checkRecipients(recipients);
             smtpMessage.addRecipients(RecipientType.TO, recipients);
             /*
@@ -642,6 +647,7 @@ public final class SMTPTransport extends MailTransport {
              * Check recipients
              */
             final Address[] recipients = allRecipients == null ? smtpMessage.getAllRecipients() : allRecipients;
+            processAddressHeader(smtpMessage);
             checkRecipients(recipients);
             try {
                 final long start = System.currentTimeMillis();
@@ -686,7 +692,7 @@ public final class SMTPTransport extends MailTransport {
             MimeMessage mimeMessage = null;
             if (composedMail instanceof ContentAware) {
                 try {
-                    Object content = composedMail.getContent();
+                    final Object content = composedMail.getContent();
                     if (content instanceof MimeMessage) {
                         mimeMessage = (MimeMessage) content;
                     }
@@ -727,6 +733,7 @@ public final class SMTPTransport extends MailTransport {
                  * Check recipients
                  */
                 final Address[] recipients = allRecipients == null ? mimeMessage.getAllRecipients() : allRecipients;
+                processAddressHeader(mimeMessage);
                 checkRecipients(recipients);
                 try {
                     final long start = System.currentTimeMillis();
@@ -774,6 +781,7 @@ public final class SMTPTransport extends MailTransport {
                 }
                 checkRecipients(recipients);
                 smtpFiller.setSendHeaders(composedMail, smtpMessage);
+                processAddressHeader(smtpMessage);
                 /*
                  * Drop special "x-original-headers" header
                  */
@@ -834,7 +842,7 @@ public final class SMTPTransport extends MailTransport {
         }
     }
 
-    private void transport(final MimeMessage smtpMessage, final Address[] recipients, Transport transport, final SMTPConfig smtpConfig) throws OXException {
+    private void transport(final MimeMessage smtpMessage, final Address[] recipients, final Transport transport, final SMTPConfig smtpConfig) throws OXException {
         prepareAddresses(recipients);
         try {
             transport.sendMessage(smtpMessage, recipients);
@@ -922,6 +930,65 @@ public final class SMTPTransport extends MailTransport {
         SMTPCapabilityCache.init();
     }
 
+    private static void processAddressHeader(final MimeMessage mimeMessage) throws OXException, MessagingException {
+        {
+            final String str = mimeMessage.getHeader("From", null);
+            if (!isEmpty(str)) {
+                final InternetAddress[] addresses = QuotedInternetAddress.parse(str, false);
+                checkRecipients(addresses);
+                mimeMessage.setFrom(addresses[0]);
+            }
+        }
+        {
+            final String str = mimeMessage.getHeader("Sender", null);
+            if (!isEmpty(str)) {
+                final InternetAddress[] addresses = QuotedInternetAddress.parse(str, false);
+                checkRecipients(addresses);
+                mimeMessage.setSender(addresses[0]);
+            }
+        }
+        {
+            final String str = mimeMessage.getHeader("To", null);
+            if (!isEmpty(str)) {
+                final InternetAddress[] addresses = QuotedInternetAddress.parse(str, false);
+                checkRecipients(addresses);
+                mimeMessage.setRecipients(RecipientType.TO, addresses);
+            }
+        }
+        {
+            final String str = mimeMessage.getHeader("Cc", null);
+            if (!isEmpty(str)) {
+                final InternetAddress[] addresses = QuotedInternetAddress.parse(str, false);
+                checkRecipients(addresses);
+                mimeMessage.setRecipients(RecipientType.CC, addresses);
+            }
+        }
+        {
+            final String str = mimeMessage.getHeader("Bcc", null);
+            if (!isEmpty(str)) {
+                final InternetAddress[] addresses = QuotedInternetAddress.parse(str, false);
+                checkRecipients(addresses);
+                mimeMessage.setRecipients(RecipientType.BCC, addresses);
+            }
+        }
+        {
+            final String str = mimeMessage.getHeader("Reply-To", null);
+            if (!isEmpty(str)) {
+                final InternetAddress[] addresses = QuotedInternetAddress.parse(str, false);
+                checkRecipients(addresses);
+                mimeMessage.setReplyTo(addresses);
+            }
+        }
+        {
+            final String str = mimeMessage.getHeader("Disposition-Notification-To", null);
+            if (!isEmpty(str)) {
+                final InternetAddress[] addresses = QuotedInternetAddress.parse(str, false);
+                checkRecipients(addresses);
+                mimeMessage.setHeader("Disposition-Notification-To", addresses[0].toString());
+            }
+        }
+    }
+
     private static void checkRecipients(final Address[] recipients) throws OXException {
         if ((recipients == null) || (recipients.length == 0)) {
             throw SMTPExceptionCode.MISSING_RECIPIENTS.create();
@@ -938,6 +1005,24 @@ public final class SMTPTransport extends MailTransport {
                 }
             }
         }
+        if (MailProperties.getInstance().isSupportMsisdnAddresses()) {
+            InternetAddress internetAddress;
+            for (final Address address : recipients) {
+                internetAddress = (InternetAddress) address;
+                final String sAddress = internetAddress.getAddress();
+                if (MsisdnCheck.checkMsisdn(sAddress)) {
+                    if (sAddress.indexOf('/') < 0) {
+                        // Detected a MSISDN address that misses "/TYPE=" appendix necessary for the MTA
+                        internetAddress.setAddress(sAddress + "/TYPE=PLMN");
+                    }
+                    try {
+                        internetAddress.setPersonal("", "US-ASCII");
+                    } catch (final UnsupportedEncodingException e) {
+                        // Ignore as personal is cleared
+                    }
+                }
+            }
+        }
     }
 
     private static final class MailCleanerTask implements Runnable {
@@ -968,12 +1053,10 @@ public final class SMTPTransport extends MailTransport {
         boolean close = false;
         final SMTPConfig config = getTransportConfig0();
         try {
-            try {
-                connectTransport(transport, config);
-                close = true;
-            } catch (final javax.mail.AuthenticationFailedException e) {
-                throw MimeMailExceptionCode.TRANSPORT_INVALID_CREDENTIALS.create(e, config.getServer(), e.getMessage());
-            }
+            connectTransport(transport, config);
+            close = true;
+        } catch (final javax.mail.AuthenticationFailedException e) {
+            throw MimeMailExceptionCode.TRANSPORT_INVALID_CREDENTIALS.create(e, config.getServer(), e.getMessage());
         } catch (final MessagingException e) {
             throw MimeMailException.handleMessagingException(e, config, session);
         } finally {
@@ -1009,6 +1092,61 @@ public final class SMTPTransport extends MailTransport {
             hostName = getHostName();
         }
         MimeMessageConverter.saveChanges(mimeMessage, hostName);
+        // Check whether to remove MIME-Version headers from sub-parts
+        if (TransportProperties.getInstance().isRemoveMimeVersionInSubParts()) {
+            /*-
+             *  Note that the MIME-Version header field is required at the top level
+             *  of a message.  It is not required for each body part of a multipart
+             *  entity.  It is required for the embedded headers of a body of type
+             *  "message/rfc822" or "message/partial" if and only if the embedded
+             *  message is itself claimed to be MIME-conformant.
+             */
+            try {
+                checkMimeVersionHeader(mimeMessage);
+            } catch (final Exception e) {
+                LOG.warn("Could not check for proper usage of \"MIME-Version\" header according to RFC2045.", e);
+            }
+        }
+    }
+
+    private void checkMimeVersionHeader(final MimeMessage mimeMessage) throws MessagingException, IOException {
+        final String header = mimeMessage.getHeader("Content-Type", null);
+        if (null != header && toLowerCase(header).startsWith("multipart/")) {
+            final Multipart multipart = (Multipart) mimeMessage.getContent();
+            final int count = multipart.getCount();
+            for (int i = 0; i < count; i++) {
+                checkMimeVersionHeader(multipart.getBodyPart(i));
+            }
+        }
+    }
+
+    private void checkMimeVersionHeader(final Part part) throws MessagingException, IOException {
+        final String[] header = part.getHeader("Content-Type");
+        if (null != header && header.length > 0 && null != header[0]) {
+            final String cts = toLowerCase(header[0]);
+            if (cts.startsWith("multipart/")) {
+                final Multipart multipart = (Multipart) part.getContent();
+                final int count = multipart.getCount();
+                for (int i = 0; i < count; i++) {
+                    checkMimeVersionHeader(multipart.getBodyPart(i));
+                }
+            } else if (cts.startsWith("message/rfc822") || cts.startsWith("message/partial")) {
+                part.setHeader("MIME-Version", "1.0");
+                Object content;
+                try {
+                    content = part.getContent();
+                } catch (Exception e) {
+                    content = null;
+                }
+                if (content instanceof MimeMessage) {
+                    checkMimeVersionHeader((MimeMessage) content);
+                }
+            } else {
+                part.removeHeader("MIME-Version");
+            }
+        } else {
+            part.removeHeader("MIME-Version");
+        }
     }
 
     private static String getHostName() {
